from slackker import callbacks
from slackker.utils import checkker
from slackker.utils import functions

__author__ = 'Siddhesh Gunjal'
__email__ = 'siddhu19@live.com'
__version__ = '1.2.33'

# module level doc-string
__doc__ = """
slackker
=====================================================================
Description
-----------
slackker is a python package for monitoring your python script & Model training status in real-time on slack & telegram.

References
----------
https://github.com/siddheshgunjal/slackker
"""